#
# Add the the necessary methods to the base class built using the
# Qt designer and pyuic -x -o cylsidesetsbase.py cylsidesetsbase.ui
#
# By deriving from the base class, changes can be made in the designer
# and propagated to the subclass without having to muck with any machine
# generated code.

from qt import *
from math import *
from cylsidesetsbase import CylSidesetsBase
import string
import cubit
import claropyqt
import broker


#
# CylSidesets implements all the behaviors that are required to make
# the wizard functional.  It is derived from the base class that is generated
# by the designer and pyuic.  The class is extended by overloading the 
# page_changed method and the accept method.
#
class CylSidesets(CylSidesetsBase):
    def __init__(self,parent = None,name = None,modal = 0,fl = 0): 
        # make sure the base class gets generated
        CylSidesetsBase.__init__(self,parent,name,modal,fl)
        
        # Create the action
        self.mAction = QAction(self);
        self.mAction.setText("Cylindrical Sidesets")                             
        self.mAction.setMenuText("&Cylindrical Sidesets")                    
        self.mAction.setStatusTip("Display Cylindrical Sideset Wizard")
        self.mAction.setToolTip("Display Cylindrical Sideset Wizard")

        # Connect the action to the show method.
        # See pyqt documentation for an explanation of signals and slots
        QObject.connect(self.mAction, SIGNAL("activated()"), self.show)

        # Insert the item in the Tools menu.
        claropyqt.add_to_menu("Wi&zards", self.mAction, "cylsidesets")

    def __del__(self):
        claropyqt.remove_menu_items("cylsidesets")
        del self.mAction
        self.mAction = None

    def show(self):
      first_page = self.page(0)
      self.showPage(first_page)
      CylSidesetsBase.show(self)


    # define the behaviors that occur when the pages change
    def page_changed(self, page_title):
        if page_title == "Define Orientation":
            cubit.cmd("display")
        elif page_title == "Finish":
            if not self.mNumberCylinders.text().isEmpty() and \
               not self.mDistance.text().isEmpty() and \
               not self.mCylinderRadius.text().isEmpty():# all the data has been entered
                   self.setFinishEnabled(self.mFinishPage, 1) 
                   self.draw_cyls()

    # this is the meat of the class.  The accept action is called when the
    # finish button is clicked.  The method here actually defines circular and
    # not oval sideset patches on a conical body.  Points are created on the 
    # surface and the points are used to define the center of the sideset patches
    # by making calls into cubit.
    def accept(self):
        # get the current sideset ids.  This assumes the sideset list is sorted
        sideset = cubit.get_sideset_id_list()
        if len(sideset) == 0:
          sideset_id = 1
        else:
          sideset_id = sideset[-1] + 1

        vect = self.origin[:] # make a shallow copy of origin

        # get the maximum possible scale for this entity for intersection calculations
        surf_id = self.mSurfaceLineEdit.text().toInt()
        bound_box = cubit.get_bounding_box("surface", surf_id[0])
        max_bb = max(bound_box)
        min_bb = min(bound_box)

        scale = max_bb - min_bb

        # get the axis of the cylinder
        if self.axis == "x":
            vect[0] = scale
            vect[1] = 0
        elif self.axis == "y":
            vect[1] = scale
            vect[2] = 0
        else:
            vect[2] = scale
            vect[0] = 0
          
        # create strings from the lists
        origin_str = ""
        for val in self.origin:
            origin_str += str(val) + " "

        vect_str = ""
        for val in vect:
            vect_str += str(val) + " "

        # get the axis defining the rotation

        # for each cylinder create a point on the surface, create a sideset 
        # around that vertex, and draw it.
        for degree in frange(0., 360., self.angle):
            axis = string.strip(self.pos_axis)
            axis = axis[1]
            command  = "create curve from location " + origin_str
            command +=  " location " + vect_str
            command +=  " swing about axis " + axis + " angle " + str(degree)
            cubit.cmd(command)
            
            # create vertex on the curve
            curve_id = cubit.get_last_id("curve")
            command = "create vertex on curve " + str(curve_id) + "  distance 0 from surface "+ str(surf_id[0])
            cubit.cmd(command)
            vertex_id = cubit.get_last_id("vertex")

            # get the coordinates of the vertex
            xyz = cubit.get_center_point("vertex", vertex_id)

            # clean up the curve that was created
            command = "delete curve " + str(curve_id)
            cubit.cmd(command)
            
            command = "sideset " + str(sideset_id) + \
              " surface " + str(surf_id[0]) + \
              " patch center " + str(xyz[0]) + " " + str(xyz[1]) + " " + str(xyz[2]) + \
              " outer_radius " + str(self.radius)
            cubit.cmd(command)

            # creates two sequential ids at both intersections.  
            # clean up the vertices
            command = "delete vertex " + str(vertex_id-1) + " " +str(vertex_id)
            cubit.cmd(command)
            sideset_id += 1
            cubit.cmd("draw sideset all add")

        # close the wizard window
        self.close()

#
# This is how Darryl solved this problem using APREPRO
#
# Define your cylinder in terms of
#  a) any point on the cylinder's axis (ox,oy,oz)
#  b) a vector defining the direction of the cylinder's axis (vx,vy,vz)
#  c) the cylinder's radius
# {ox=1}
# {oy=1}
# {oz=0}
# {vx=0}
# {vy=0}
# {vz=1}
# {rad=3}

# Normalize the axis vector
# {v_mag=sqrt(vx*vx + vy*vy + vz*vz)}
# {vx/=v_mag}
# {vy/=v_mag}
# {vz/=v_mag}

# create a mesh for demonstration purposes
#bri x 10
#vol 1 inter 10
#mesh vol 1

# Add to a group all faces whose centers lie inside of the cylinder.
# Note that the cylinder is infinite.
# In case you couldn't tell, this line is the key to your problem...
#group "intsct_faces" add face with (x_coord - {ox} - ((x_coord - {ox})*{vx} + (y_coord - {oy})*{vy} + (z_coord - {oz})*{vz})*{vx})*(x_coord - {ox} - ((x_coord - {ox})*{vx} + (y_coord - {oy})*{vy} + (z_coord - {oz})*{vz})*{vx}) + (y_coord - {oy} - ((x_coord - {ox})*{vx} + (y_coord - {oy})*{vy} + (z_coord - {oz})*{vz})*{vy})*(y_coord - {oy} - ((x_coord - {ox})*{vx} + (y_coord - {oy})*{vy} + (z_coord - {oz})*{vz})*{vy}) + (z_coord - {oz} - ((x_coord - {ox})*{vx} + (y_coord - {oy})*{vy} + (z_coord - {oz})*{vz})*{vz})*(z_coord - {oz} - ((x_coord - {ox})*{vx} + (y_coord - {oy})*{vy} + (z_coord - {oz})*{vz})*{vz}) < {rad}*{rad}


    def draw_cyls(self):
        num_cyls    = self.mNumberCylinders.text().toInt()
        dist   = self.mDistance.text().toFloat()
        self.radius = self.mCylinderRadius.text().toFloat()[0] 
        pos_axis_id = self.mPositionButtonGroup.selectedId() 
        rot_axis_id = self.mAxisButtonGroup.selectedId() 

        if pos_axis_id == 0:
            self.pos_axis = " Ax "
            dist_str =  str(dist[0]) + " 0 0" 
            self.origin = [dist[0], 0., 0.]
        elif pos_axis_id == 1:
            self.pos_axis = " Ay "
            dist_str = "0 "+ str(dist[0]) + " 0"
            self.origin = [0., dist[0], 0.]
        else:
            self.pos_axis = " Az "
            dist_str = "0 0 " + str(dist[0])
            self.origin = [0., 0., dist[0]]

        if rot_axis_id == 0:
            self.axis = "x"
        elif rot_axis_id == 1:
            self.axis = "y"
        else:
            self.axis = "z"
   
        self.angle = 180.0/num_cyls[0]
        for degree in frange(0., 180., self.angle):
            command = "draw cylinder radius " + str(self.radius) + \
                      " axis " + self.axis + " rotate " + self.pos_axis + str(degree) + \
                      " center "+ dist_str + " extended percentage 25"
            cubit.cmd(command)


# Python ranges don't handle floats by default.  Correct that deficiency.
def frange(start, end=None, inc=None):
    "A range function, that does accept float increments..."

    if end == None:
        end = start + 0.0
        start = 0.0
    else: start += 0.0 # force it to be a float

    if inc == None:
        inc = 1.0

    count = int(ceil((end-start)/inc))

    L = [None,] * count
    for i in xrange(count):
        L[i] = start + i * inc

    return L 

# This class defines this object as a component that can be loaded into the
# claro framework.
class CylSidesetsComponent:
    def __init__(self):
        # keep this around as needed
        self.cyl_sidesets = None

    def start_up(self, with_gui):
        if with_gui > 0:
            self.cyl_sidesets = CylSidesets(qApp.mainWidget())

    def clean_up(self):

        # remove the reference to this object held by Qt
        qApp.mainWidget().removeChild(self.cyl_sidesets)

        # delete the object
        del self.cyl_sidesets

        # let the broker know we have cleaned up
        broker.clean_up_complete('cylsidesets')
        
    def interrupt_progress(self):
        pass


