#
# Add the the necessary methods to the base class built using the
# Qt designer and pyuic -x -o cylsidesetsbase.py cylsidesetsbase.ui
#
# By deriving from the base class, changes can be made in the designer
# and propagated to the subclass without having to muck with any machine
# generated code.

import os
from qt import *
from math import *
from planesidesetsbase import PlaneSidesetsBase
import broker
import string
import claropyqt
import cubit

#
# PlaneSidesets implements all the behaviors that are required to make
# the wizard functional.  It is derived from the base class that is generated
# by the designer and pyuic.  The class is extended by overloading the 
# page_changed method and the accept method.
#
class PlaneSidesets(PlaneSidesetsBase):
    def __init__(self,parent = None,name = None,modal = 0,fl = 0): 
        # make sure the base class gets generated
        PlaneSidesetsBase.__init__(self,parent,name,modal,fl)

        # Create the action
        self.customAction = QAction(self)                               
        self.customAction.setText("Planar Sidesets")                             
        self.customAction.setMenuText("P&lanar Sidesets ...")                    
        self.customAction.setOn(0)
        self.customAction.setStatusTip("Display Planar Sideset Wizard")
        self.customAction.setToolTip("Display Planar Sideset Wizard")
        self.customAction.setAccel(QAccel.stringToKey("CTRL+Y"),)

        #connect the activated signal to the display 
        QObject.connect(self.customAction, SIGNAL("activated()"), self.show)

        #Insert the popup into the menubar
        claropyqt.add_to_menu("Custom", self.customAction, "custom")

        self.end_dist = 0
        self.origin_id = []
        self.endpoint_id = []
        self.curve_id = []

    def __del__(self):
        claropyqt.remove_menu_items("custom")

    def show(self):
      first_page = self.page(0)
      self.showPage(first_page)
      PlaneSidesetsBase.show(self)


    # define the behaviors that occur when the pages change
    def page_changed(self, page_title):
        filled_out = 0 # assume the iteration page does not have all fields filled

        if page_title == "Define Orientation":
            cubit.cmd("display")
        elif page_title == "Finish":
            # check the iteration page first
            if self.mStartDistance.text().isEmpty() or \
               self.mNumSteps.text().isEmpty():
                pass
            elif self.mNumSteps.text().toFloat()[0] == 0:
                filled_out = 1
            elif not self.mEndDistance.text().isEmpty():
                filled_out = 1
            else:
                print "ERROR: logic error in page_changed"
            
            if filled_out == 1 and \
              not self.mNormalX.text().isEmpty() and \
              not self.mNormalY.text().isEmpty() and \
              not self.mNormalZ.text().isEmpty():# all the data has been entered
                self.setFinishEnabled(self.mFinishPage, 1) 
                self.draw_plane()
            else:
                self.setFinishEnabled(self.mFinishPage, 0) 

    def draw_plane(self):

        origin = [0,0,0]
        pos_axis_id = self.mPositionButtonGroup.selectedId() 

        self.start_dist = self.mStartDistance.text().toFloat()[0]

        self.steps = self.mNumSteps.text().toFloat()[0]

        if self.steps > 0:
            self.end_dist = self.mEndDistance.text().toFloat()[0]

        self.vx = self.mNormalX.text().toFloat()[0]
        self.vy = self.mNormalY.text().toFloat()[0]
        self.vz = self.mNormalZ.text().toFloat()[0]

        if self.steps == 0:
            self.steps = 1
        else:
            self.steps = (self.end_dist - self.start_dist)/self.steps

        for dist in frange(self.start_dist, self.end_dist, self.steps):

            if pos_axis_id == 0:
                origin[0] = dist
            elif pos_axis_id == 1:
                origin[1] = dist
            else:
                origin[2] = dist

            endpoint = []
            endpoint[:] = origin

            endpoint[0] += self.vx
            endpoint[1] += self.vy
            endpoint[2] += self.vz

            # create a vertex at the origin
            origin_str = self.list_to_str(origin)

            command = "create vertex " + origin_str
            print "create origin command: ",command
            cubit.cmd(command)
            self.origin_id.append(cubit.get_last_id("vertex"))

            # create the endpoint vertex
            endpoint_str = self.list_to_str(endpoint)

            command = "create vertex " + endpoint_str
            print "create endpoint command: ",command
            cubit.cmd(command)
            last_vertex = cubit.get_last_id("vertex")
            self.endpoint_id.append(last_vertex)

            # create the curve between the vertices
            command = "create curve " + str(self.origin_id[-1]) + " " + str(self.endpoint_id[-1])
            print "create curve command: ",command
            cubit.cmd(command)
            self.curve_id.append(cubit.get_last_id("curve"))

            # draw the plane normal to the curve
            command = "draw plane normal to curve " + str(self.curve_id[-1]) + " fraction 0 add"
            cubit.cmd(command)


    def make_sidesets(self,  offset):
        mag = sqrt(self.vx*self.vx + self.vy*self.vy + self.vz*self.vz)

        x = self.vx/mag
        y = self.vy/mag
        z = self.vz/mag

        xcoord = " x_coord*" + str(x) + " + "
        ycoord = " y_coord*" + str(y) + " + "
        zcoord = " z_coord*" + str(z)
        command = "sideset 1 face with " + xcoord + ycoord + zcoord + " > " + str(offset)
        cubit.cmd(command)
        command = "sideset 2 face with " + xcoord + ycoord + zcoord + " < " + str(offset)
        cubit.cmd(command)
    
    def enable_end_distance(self, text):
        if text.toInt()[0] >  0:
            self.mEndDistance.setEnabled(1);
        else:
            self.mEndDistance.setEnabled(0);
        

    def browse_pressed(self):
        dir = QFileDialog.getExistingDirectory( "", self, "Select output directory", \
                   "Select output directory", 1 )

        if not dir.isEmpty():
            self.mDirectory.setText(dir)
        

    def accept(self):

      if self.mExportExodus.isChecked():
        if self.mDirectory.text().isEmpty():
          QMessageBox.warning( self, "Plane Sidesets",
                    "An output directory must be specified.")
          return
        elif self.mBaseName.text().isEmpty():
          QMessageBox.warning( self, "Plane Sidesets",
                    "A base name must be specified")
          return
        else:
          # change working directories
          cwd = os.getcwd()
          command = "cd " + "'" + self.mDirectory.text().ascii() + "'"
          #try:
          #  command = "cd " + "'" + self.mDirectory.text().ascii() + "'"
          #except:
          #  print 'error'
          #  import traceback
          #  traceback.print_exc()
          cubit.cmd(command)

          base_name = self.mBaseName.text().ascii()
          i = 1
          print "Before for loop"
          for dist in frange(self.start_dist, self.end_dist, self.steps):
            print "in for loop"
            # make the sidesets
            self.make_sidesets(dist)

            # write the files
            name = base_name + "_" + str(i) + ".g"
            i = i + 1
            command = "export mesh " + "'" + name + "'" + " overwrite"
            print "Export command: ", command
            cubit.cmd(command)

            # display
            cubit.cmd("draw sideset 1 2")

            # clean up the sidesets
            cubit.cmd("delete sideset 1")
            cubit.cmd("delete sideset 2")

          # reset the working directory
          command = "cd " + "'" + cwd + "'"
          print command
          cubit.cmd(command)

      else:
        self.make_sidesets(self.start_dist)

      # clean up
      curve_str = self.list_to_str(self.curve_id)
      cubit.cmd("delete curve " + curve_str)
      origin_str = self.list_to_str(self.origin_id)
      cubit.cmd("delete vertex " + origin_str)
      endpoint_str = self.list_to_str(self.endpoint_id)
      cubit.cmd("delete vertex " + endpoint_str)

      self.close()

    # convert a list of the form [1,2] (with square brackets) to a string without the []
    def list_to_str(self, alist):
      alist_str = str(alist)
      alist_str = alist_str.replace('[','')
      alist_str = alist_str.replace(']','')

      return alist_str
    

# Python ranges don't handle floats by default.  Correct that deficiency.
def frange(start, end=None, inc=None):
    "A range function, that does accept float increments..."

    if end == None:
        end = start + 0.0
        start = 0.0
    else: start += 0.0 # force it to be a float

    if inc == None:
        inc = 1.0

    count = int(ceil((end-start)/inc))

    L = [None,] * count
    for i in xrange(count):
        L[i] = start + i * inc

    return L 

# This class defines this object as a component that can be loaded into the
# claro framework.
class PlaneSidesetsComponent:
    def __init__(self):
        # keep this around as needed
        self.plane_sidesets = None

    def start_up(self, with_gui):
        if with_gui > 0:
            self.plane_sidesets = PlaneSidesets(qApp.mainWidget())

    def clean_up(self):

        # remove the reference to this object held by Qt
        qApp.mainWidget().removeChild(self.plane_sidesets)

        # delete the object
        del self.plane_sidesets

        # let the broker know we have cleaned up
        broker.clean_up_complete('planesidesets')
        
    def interrupt_progress(self):
        pass


