#
# Add the the necessary methods to the base class built using the
# Qt designer and pyuic -x -o teststrapbase.py teststrapbase.ui
#
# By deriving from the base class, changes can be made in the designer
# and propagated to the subclass without having to muck with any machine
# generated code.

from qt import *
from math import *
from teststrapbase import TestStrapBase
import broker
import string
import claropyqt
import cubit

#
# TestStrap implements all the behaviors that are required to make
# the wizard functional.  It is derived from the base class that is generated
# by the designer and pyuic.  The class is extended by overloading the 
# page_changed method and the accept method.
#
class TestStrap(TestStrapBase):
  def __init__(self,parent = None,name = None,modal = 0,fl = 0): 
    # make sure the base class gets generated
    TestStrapBase.__init__(self,parent,name,modal,fl)

    # Create the action
    self.customAction=QAction(self)                               
    self.customAction.setText("Parametric Test Strap")                             
    self.customAction.setMenuText("Parametric Test Strap")                    
    self.customAction.setOn(0)
    self.customAction.setStatusTip("Parametric Test Strap Dialog")
    self.customAction.setToolTip("Parametric Test Strap Dialog")

    #connect the activated signal to the display 
    QObject.connect(self.customAction, SIGNAL("activated()"), self.show)

    #Insert the popup into the menubar
    claropyqt.add_to_menu("Custom", self.customAction, "custom")

  def __del__(self):
    claropyqt.remove_menu_items("custom")

  # The accept action is called when the OK button is clicked. 
  def accept(self):
    # make sure all the necessary data is there
    if self.r1.text().isEmpty() or self.r2.text().isEmpty() or self.r3.text().isEmpty() or \
       self.l1.text().isEmpty() or self.l2.text().isEmpty() or \
       self.l3.text().isEmpty() or self.l4.text().isEmpty() or \
       self.l5.text().isEmpty() or self.l6.text().isEmpty() or \
       self.t.text().isEmpty():
      QMessageBox.warning( self, "Parametric Test Strap",
                  "All data fields must be filled")
      return

    # get all the values from the dialog
    r1 = self.r1.text().toFloat()[0]
    r2 = self.r2.text().toFloat()[0]
    r3 = self.r3.text().toFloat()[0]

    l1 = self.l1.text().toFloat()[0]
    l2 = self.l2.text().toFloat()[0]
    l3 = self.l3.text().toFloat()[0]
    l4 = self.l4.text().toFloat()[0]
    l5 = self.l5.text().toFloat()[0]
    l6 = self.l6.text().toFloat()[0]

    thickness = self.t.text().toFloat()[0]

    # check the constraints on the model
    if l2 < (l1 + r1):
      QMessageBox.warning( self, "Parametric Test Strap",
                    "L2 must be greater than L1 + R1")
      return

    if (l3 + r2) < r1:
      QMessageBox.warning( self, "Parametric Test Strap",
                    "L3 + R2 must be greater than R1")
      return

    if l6 > r2 + r3 + l4 + l5 - r1 -l2 + l1:
      QMessageBox.warning( self, "Parametric Test Strap",
                    "L6 must be less than R2 + R3 -R1 + L1 + L4 + L5 - L2")
      return
#
# Interface to an APREPRO script in cubit load the python variables
# into APREPRO
#
    command = "#{R1 = " + str(r1) + "}"
    cubit.cmd(command)
    command = "#{R2 = " + str(r2) + "}"
    cubit.cmd(command)
    command = "#{R3 = " + str(r3) + "}"
    cubit.cmd(command)
    command = "#{L1 = " + str(l1) + "}"
    cubit.cmd(command)
    command = "#{L2 = " + str(l2) + "}"
    cubit.cmd(command)
    command = "#{L3 = " + str(l3) + "}"
    cubit.cmd(command)
    command = "#{L4 = " + str(l4) + "}"
    cubit.cmd(command)
    command = "#{L5 = " + str(l5) + "}"
    cubit.cmd(command)
    command = "#{L6 = " + str(l6) + "}"
    cubit.cmd(command)
    command = "#{T = " + str(thickness) + "}"
    cubit.cmd(command)
#
# Create the command scrpt
#
    cubit.cmd("create vertex {L1} 0 0")
    cubit.cmd("create vertex {L1+R1} 0 0")
    cubit.cmd("create vertex {L1} {R1} 0")
    cubit.cmd("create vertex {L2} 0 0")
    cubit.cmd("create vertex {L2} {L3} 0")
    cubit.cmd("create vertex {L2-R2} {L3} 0")
    cubit.cmd("create vertex {L2-R2} {L3+R2}")
    cubit.cmd("create vertex {L2-R2-L4} {L3+R2}")
    cubit.cmd("create vertex {L2-R2-L4} {L3+R2+R3}")
    cubit.cmd("create vertex {L2-R2-L4-R3} {L3+R2+R3}")
    cubit.cmd("create vertex {L2-R2-L4-R3-L5} {L3+R2+R3}")
    cubit.cmd("create vertex {L2-R2-L4-R3-L5} 0")
    cubit.cmd("create vertex {L1-L6} {R1}")
    cubit.cmd("create vertex {L1-L6} 0")
    cubit.cmd("create vertex {L1-L6-R1} 0")
    cubit.cmd("#")
    cubit.cmd("# Create the arcs")
    cubit.cmd("#")
    cubit.cmd("create curve arc center vertex 1 2 3")
    cubit.cmd("create curve arc center vertex 14 13 15")
    cubit.cmd("create curve arc center vertex 6 5 7")
    cubit.cmd("create curve arc center vertex 9 10 8")
    cubit.cmd("#")
    cubit.cmd("# Create the lines")
    cubit.cmd("#")
    cubit.cmd("create curve vertex 3 vertex 13")
    cubit.cmd("create curve vertex 4 vertex 5")
    cubit.cmd("create curve vertex 7 vertex 8")
    cubit.cmd("create curve vertex 10 vertex 11")
    cubit.cmd("create curve vertex 11 vertex 12")
    cubit.cmd("#")
    cubit.cmd("# Reflect the curves and cleanup construction points")
    cubit.cmd("#")
    cubit.cmd("curve all copy reflect y")
    cubit.cmd("merge curve all")
    cubit.cmd("delete vertex all")
    cubit.cmd("#")
    cubit.cmd("# Create surface")
    cubit.cmd("#")
    cubit.cmd("create surface curve all")
    cubit.cmd("#")
    cubit.cmd("# Create Volumes")
    cubit.cmd("#")
    cubit.cmd("sweep surface all perpendicular distance {T}")
    cubit.cmd("create cylinder height 3 radius {R1}")
    cubit.cmd("move volume 2 x {L1} z {T/2}")
    cubit.cmd("#")
    cubit.cmd("# Mesh to volumes")
    cubit.cmd("#")
    cubit.cmd("volume all size {T/2}")
    cubit.cmd("mesh volume all")
    cubit.cmd("#")
    cubit.cmd("# Set up the BCs (as well as cubit can)")
    cubit.cmd("#")
    cubit.cmd("nodeset 1 surface 13 14 ")
    cubit.cmd("sideset 1 surface 2 7 ")
    cubit.cmd("sideset 2 face in surf 21 with x_coord > {L1-.001}")
                  
# This class defines this object as a component that can be loaded into the
# claro framework.
class TestStrapComponent:
    def __init__(self):
        # keep this around as needed
        self.teststrap = None

    def start_up(self, with_gui):
        if with_gui > 0:
            self.teststrap = TestStrap(qApp.mainWidget())

    def clean_up(self):

        # remove the reference to this object held by Qt
        qApp.mainWidget().removeChild(self.teststrap)

        # delete the object
        del self.teststrap

        # let the broker know we have cleaned up
        broker.clean_up_complete('teststrap')
        
    def interrupt_progress(self):
        pass

