#
# Add the the necessary methods to the base class built using the
# Qt designer and pyuic -x -o simpleclassbase.py simpleclassbase.ui
#
# By deriving from the base class, changes can be made in the designer
# and propagated to the subclass without having to muck with any machine
# generated code.

from qt import *
from simpleclassbase import SimpleClassBase
import claropyqt
import broker


#
# This very simple class is derived from the base class that is generated
# by the designer and pyuic.  The class is extended by overloading the 
# pbClose_clicked() method.
#

# Notice the name includes "Component" at the end. Claro will look for this
# class when attempting to load the component.

class SimpleClass(SimpleClassBase):
    def __init__(self,parent = None,name = None, fl = 0): 

        # make sure the base class gets generated
        SimpleClassBase.__init__(self,parent,name, fl)

        # Create the action - place it into Claro's menu bar
        self.mAction = QAction(self);
        self.mAction.setText("Simple Class")                             
        self.mAction.setMenuText("Simple")                    
        self.mAction.setStatusTip("Display the Simple Widget")
        self.mAction.setToolTip("Display the Simple Widget")

        # Connect the action to the show method.
        # See pyqt documentation for an explanation of signals and slots
        QObject.connect(self.mAction, SIGNAL("activated()"), self.show)

        # Insert the item in the 'Tools' menu in Claro
        claropyqt.add_to_menu("Simple Class", self.mAction, "simpleclass")

    def __del__(self):
        claropyqt.remove_menu_items("simpleclass")
        del self.mAction
        self.mAction = None

    def show(self):
        # position the widget in the middle of the application so we can see it
        self.setGeometry(500,300, 200, 200)
        SimpleClassBase.show(self)


    # The pbClose_clicked() method is called when the 'Close' button is pushed. 
    # Look into the base class and you will see the stub for this method.

    def pbClose_clicked(self):
        # Close the widget
        self.close()

class SimpleClassComponent:
    def __init__(self):
        # keep this around as needed
        self.simple_class = None

    # This method is required by Claro for all components
    def start_up(self, with_gui):
        if with_gui > 0:
            self.simple_class = SimpleClass(qApp.mainWidget())

    # This method is required by Claro for all components
    def clean_up(self):

        # remove the reference to this object held by Qt
        qApp.mainWidget().removeChild(self.simple_class)

        # delete the object
        del self.simple_class

        # let the broker know we have cleaned up
        broker.clean_up_complete('simpleclass')
        
    # This method is required by Claro for all components
    def interrupt_progress(self):
        pass
