
#
# Add the the necessary methods to the base class built using the
# Qt designer and pyuic -x -o elimsidesetsbase.py elimsidesetsbase.ui
#
# By deriving from the base class, changes can be made in the designer
# and propagated to the subclass without having to muck with any machine
# generated code.

from qt import *
from elimsidesetsbase import ElimSidesetsBase
import string
import claropyqt
import broker
import cubit


class ElimSidesets(ElimSidesetsBase):
  def __init__(self,parent = None,name = None,modal = 0,fl = 0): 
    # make sure the base class gets generated
    ElimSidesetsBase.__init__(self,parent,name,modal,fl)
      
    # Create the action
    self.mAction = QAction(self)
    self.mAction.setText("Eliminate Sidesets")                             
    self.mAction.setMenuText("Eliminate Sidesets")                    
    self.mAction.setStatusTip("Eliminate 0 Volume Sidesets")
    self.mAction.setToolTip("Eliminate 0 Volume Sidesets")

    # Connect the action to the show method.
    QObject.connect(self.mAction, SIGNAL("activated()"), self.show)

    # Insert the item in the Tools menu.
    claropyqt.add_to_menu("Thermal", self.mAction, "elimsidesets")

    # create a popup menu for the list box
    self.popup = QPopupMenu(self.lvSideset)
    self.popup.insertItem("Draw", self.draw_sideset)
    self.popup.insertItem("Fly-to", self.fly_to_sideset)
    self.popup.insertItem("Delete", self.delete_sideset)

    # initialize class variables
    self.bad_sideset_list = []
    self.list_items = []
    self.lvSideset.setSorting(1)
    
    QObject.connect(self.lvSideset,SIGNAL('contextMenuRequested(QListViewItem *, const QPoint &, int)'),
                self.context_menu)

  def __del__(self):
    claropyqt.remove_menu_items("elimsidesets")
    del self.mAction
    self.mAction = None

  def context_menu(self, item, point, val):
    if item != None:
      self.popup.exec_loop( point)

  def draw_sideset(self):
    item = self.lvSideset.currentItem()
    command = "draw sideset " + item.text(0).ascii()
    cubit.silent_cmd(command)
    cubit.silent_cmd("zoom reset")

  def fly_to_sideset(self):
    item = self.lvSideset.currentItem()
    sideset = int(item.text(0).ascii())
    faces    = cubit.vectori()
    surfaces = cubit.vectori()
    curves   = cubit.vectori()
    cubit.get_sideset_children(sideset, faces, surfaces, curves)

    surf_id = str(surfaces[0])
    command = "zoom surface " + surf_id + " animation steps 15 direction on surf " + surf_id + " reverse"
    cubit.silent_cmd(command)

  def delete_sideset(self):
    item = self.lvSideset.currentItem()
    command = "delete sideset " + item.text(0).ascii()
    cubit.cmd(command)
    self.lvSideset.takeItem(item)

  def show(self):
    ElimSidesetsBase.show(self)

# analyze the model by attempting to create volumes from facet
# based surfaces.  If the volume is less than tolerance then 
# this is a bad group of sidesets
  def analyze(self):
    # clear the QListView at the start of the analysis
    self.lvSideset.clear()

    # initialize the error count to zero
    error_count = 0

    # create std::vector<int> to store face, surface, and curve ids.
    faces    = cubit.vectori()
    surfaces = cubit.vectori()
    curves   = cubit.vectori()
    volume   = cubit.vectori()

    tol = self.mTolerance.text().toDouble()
    if tol[1] == 0:
      # should add a message box
      return
      
    self.bad_sideset_list = []

    # get the sidesets in the model
    sidesets = cubit.get_sideset_id_list()
    self.progressBar.setTotalSteps(len(sidesets))
    progress = 0

    for s in sidesets:
      # set the progress bar
      progress = progress+1

      # get the sideset's components
      cubit.get_sideset_children(s, faces, surfaces, curves)
      if len(faces) != 0 or len(curves) != 0:
        print "Skipping sideset ", s, ". Contains faces or curves"
        self.progressBar.setProgress(i)
        continue

      if len(surfaces) == 0:
        self.progressBar.setProgress(progress)
        faces.clear()
        curves.clear()
        surfaces.clear()
        continue

      # can't create a volume with one surface (unless it's a sphere in MBG?)
      if len(surfaces) == 1:
        self.progressBar.setProgress(progress)
        faces.clear()
        curves.clear()
        surfaces.clear()
        continue

      # attempt to create a volume from the surfaces
      command = "create volume surface "
      for i in surfaces:
        command = command + str(i) + " "
      command = command + " noheal keep"

      cubit.silent_cmd(command)

      # check the error state from the create
      if cubit.get_error_count() > error_count:
        error_count = cubit.get_error_count()
        print "Error creating volume ", cubit.get_last_id("volume")
      #  self.progressBar.setProgress(progress)
      #  faces.clear()
      #  curves.clear()
      #  surfaces.clear()
      #  continue

      # get the "volume of the volume"
      volume.append(cubit.get_last_id("volume"))
      vol = cubit.get_total_volume(volume)

      # check for a bad volume
      if vol < tol[0]:
        self.bad_sideset_list.append(s)
        self.list_items.append(QListViewItem(self.lvSideset, str(s), str(vol)))

      # clean up
      cubit.silent_cmd("delete vol " + str(volume[0]))
      if len(volume) != 1:
        print "multiple volumes: "
        for i in volume:
          print i + " "
        exit
      if cubit.get_error_count() > error_count:
        error_count = cubit.get_error_count()
        print "Error deleting volume ", volume[0]
      cubit.cmd(command)
      faces.clear()
      curves.clear()
      surfaces.clear()
      volume.clear()


      # increment the progress bar
      self.progressBar.setProgress(progress)
     
    # done with loop reset progress bar
    self.progressBar.reset()
    
# Python ranges don't handle floats by default.  Correct that deficiency.
def frange(start, end=None, inc=None):
  "A range function, that does accept float increments..."

  if end == None:
    end = start + 0.0
    start = 0.0
  else: start += 0.0 # force it to be a float

  if inc == None:
    inc = 1.0

  count = int(ceil((end-start)/inc))

  L = [None,] * count
  for i in xrange(count):
    L[i] = start + i * inc

  return L 

# This class defines this object as a component that can be loaded into the
# claro framework.
class ElimSidesetsComponent:
  def __init__(self):
    # keep this around as needed
    self.elimsidesets = None

  def start_up(self, with_gui):
    if with_gui > 0:
      self.elimsidesets = ElimSidesets(qApp.mainWidget())

  def clean_up(self):

    # remove the reference to this object held by Qt
    qApp.mainWidget().removeChild(self.elimsidesets)

    # delete the object
    del self.elimsidesets

    # let the broker know we have cleaned up
    broker.clean_up_complete('elimsidesets')
        
  def interrupt_progress(self):
    pass


