#!python
#
# This script will create minimum length bar elements between two surfaces.
# The user sets the input surfaces and the generated block containing the
# bar elements.
#
import cubit

# SET YOUR SURFACE ID's HERE. Bars are propagated from 1 to 2 (order matters)
surface1 = 10
surface2 = 6
block_id = 100 # The generated bars are placed in this block

# NO CHANGES SHOULD BE NEEDED AFTER THIS POINT

# place the nodes in a group
cubit.cmd('group "surface1" add node in surface ' + str(surface1))
cubit.cmd('group "surface2" add node in surface ' + str(surface2))

# get the group ids
group1 = cubit.get_id_from_name("surface1")
group2 = cubit.get_id_from_name("surface2")

# get the nodes in each group
nodes1 = cubit.get_group_nodes(group1)
nodes2 = cubit.get_group_nodes(group2)

# initialize the minimum distance to be an arbitrary large number
min_dist = 1e+20

# Save the first generated curve for block generation
first_curve_id = cubit.get_last_id("curve") + 1

# this must be an n^2 search because it is a one-to-many search

for node1_id in nodes1:
   vec1 = cubit.get_nodal_coordinates(node1_id)
   for node2_id in nodes2:
      vec2 = cubit.get_nodal_coordinates(node2_id)
#
      x = vec1[0] - vec2[0]
      y = vec1[1] - vec2[1]
      z = vec1[2] - vec2[2]
#
      # calculate the distance between the nodes and 
      # save the data if is less than the current minimum distance
      dist_sqr = x*x + y*y + z*z
      if (dist_sqr < min_dist):
         min_dist = dist_sqr
         end_node = node2_id
#
   # found the closest node for this pair now build a bar between the nodes
   command = "create curve location node " + str(node1_id) + " location node " + str(end_node)
   cubit.cmd(command)
   curve_id = cubit.get_last_id("curve")
   command = "create edge node " + str(node1_id) + "  node " + str(end_node) + " owner curve " + str(curve_id)
   cubit.cmd(command)
   min_dist = 1e+20


# create a block with the generated bars
command = "block " + str(block_id) + " curve " + str(first_curve_id) + " to " + str(curve_id)
cubit.cmd(command)

# clean up temporary groups
cubit.cmd("delete group surface1")
cubit.cmd("delete group surface2")


